#!/usr/bin/php
<?php

error_reporting(E_ALL);
ini_set('display_errors', 'on');

function usage() {
	echo <<<'EOD'
Usage:
    vcfdiff COMMAND [ARGUMENTS...]

Commands:
    vcfdiff parse file1.vcf
	
	vcfdiff diff file1.vcf file2.vcf
     Set VCFDIFF_NORMALISE=1 in environment to strip photos/prefs when parsing.
	
EOD;
	die(1);
}

define('SHOULD_NORMALISE', getenv("VCFDIFF_NORMALISE") == "1");

class VCFEntry {
	public $pairs = [];
	
	public static function FromVCF($data) {
		$lines = explode("\n", $data);
		$last_k = null;
		$ret = new VCFEntry();
		
		foreach($lines as $line) {
			$line = str_replace("\r", "", $line);
			if (strlen($line) === 0) {
				continue;
			}
			
			if ($line[0] == " " || $line[0] == "=" /* quoted-printable */) {
				$ret->pairs[$last_k] .= substr($line, 1);
				
			} else {
				list($k, $v) = explode(":", $line, 2);
				$ret->pairs[$k] = $v;
				$last_k = $k;
			}
		}
		
		ksort($ret->pairs);
		
		if (SHOULD_NORMALISE) {
			foreach($ret->pairs as $k => $_) {
				if (strpos($k, 'PHOTO') === 0) {
					unset($ret->pairs[$k]);
				}
			}
		}
		
		return $ret;
	}
	
	public function DisplayName() {
		if (array_key_exists('FN', $this->pairs)) {
			return $this->pairs['FN'];
		} else if (array_key_exists('N', $this->pairs)) {
			return $this->pairs['N'];
		} else {
			return substr(json_encode($this->pairs), 0, 30).'...';
		}
	}
	
	public function Hash() {
		return sha1(json_encode($this->pairs), false);
	}
}

class VCFFile {
	/**
	 * @var \VCFEntry[]
	 */
	public $entries = [];
	
	public $hash_lookup = [];
	
	public static function FromFile($filename) {
		$ret = new VCFFile();
		$ret->entries = [];
		
		$data = file_get_contents($filename);
		if ($data === false) {
			die(1);
		}
		
		$vcards = [];
		preg_match_all("~BEGIN:VCARD\r?\n(.+?)END:VCARD\r?\n~ms", $data, $vcards, PREG_SET_ORDER);
		foreach($vcards as $vcard) {
			$e = VCFEntry::FromVCF( $vcard[1] );
			
			$ret->entries[] = $e;
			$ret->hash_lookup[ $e->Hash() ] = count($ret->entries) - 1;
		}
		
		ksort($ret->hash_lookup);
		
		return $ret;
	}
	
	public function EntryCount() {
		return count($this->entries);
	}	
}

function vcf_cmd_parse($filename) {
	$vcf = VCFFile::FromFile( $filename );
	echo json_encode($vcf->entries, JSON_PRETTY_PRINT);	
	return true;
}

function vcf_cmd_diff($filea, $fileb) {
	$a = VCFFile::FromFile($filea);
	$b = VCFFile::FromFile($fileb);
	
	$diff_a = array_diff(array_keys($a->hash_lookup), array_keys($b->hash_lookup));
	$diff_b = array_diff(array_keys($b->hash_lookup), array_keys($a->hash_lookup));
	
	echo sprintf("Found %d / %d entries\n\n", $a->EntryCount(), $b->EntryCount());
	
	//
	
	echo "Found ".count($diff_a)." entries appearing in '${filea}' but NOT in '${fileb}':\n\n";
	$names = [];
	foreach($diff_a as $hash) {
		$names[] = "- ".$a->entries[$a->hash_lookup[$hash]]->DisplayName()."\n";
	}
	sort($names);
	echo implode($names)."\n\n";
	
	//
	
	echo "Found ".count($diff_b)." entries appearing in '${fileb}' but NOT in '${filea}':\n\n";
	$names = [];
	foreach($diff_b as $hash) {
		$names[] = "- ".$b->entries[$b->hash_lookup[$hash]]->DisplayName()."\n";
	}
	sort($names);
	echo implode($names)."\n\n";
		
		
	//
	
	return true;
}

function main(array $args) {
	if (count($args) < 2) {
		return usage();
	}
	
	switch($args[1]) {
		case 'parse': {
			if (count($args) != 3) {
				return usage();
			}
			return vcf_cmd_parse($args[2]);			
		} break;
		
		case 'diff': {
			if (count($args) != 4) {
				return usage();
			}
			return vcf_cmd_diff($args[2], $args[3]);
		} break;
		
		default:
			usage();
	}
	
}

die(main($_SERVER['argv']) ? 0 : 1);
